#include "mpris_connection.hpp"
#include <exception>
#include <iterator>
#include <sdbus-c++/IConnection.h>
#include <sdbus-c++/IObject.h>
#include <sdbus-c++/IProxy.h>
#include <sdbus-c++/Types.h>
#define IMGUI_DEFINE_MATH_OPERATORS
#include "raylib.h"
#include "raymath.h"
#include "rlgl.h"
#include <optional>
#include <unordered_map>
#include <vector>

#include "imgui.h"
#include "imgui_stdlib.h"
#include "rlImGui/rlImGui.h"

#include "testcpp.hpp"

#include "stdio.h"

#include <curl/curl.h>
#include <fstream>
#include <iostream>
#include <rlgl.h>
#include <sdbus-c++/sdbus-c++.h>
#include <string>
extern "C" {
#include "testc.h"
}

class RayWindow {
  public:
    bool Open = false;

    RenderTexture ViewTexture;

    virtual void Setup() = 0;
    virtual void Shutdown() = 0;
    virtual void Show() = 0;
    virtual void Update() = 0;

    bool Focused = false;

    Rectangle ContentRect = {0};
};

class RayWindowExample {
  public:
    RenderTexture ViewTexture;
    bool Open = true;
    bool Focused = false;
    bool Resized = false;
    ImVec2 PastSize;
    RayWindowExample() {
        ViewTexture = LoadRenderTexture(GetScreenWidth(), GetScreenWidth());
        // ViewTexture = LoadRenderTexture(256, 256);
        printf("window w,h: %d, %d \n", ViewTexture.texture.width, ViewTexture.texture.height);
        PastSize = ImVec2{256, 256};
        Resized = false;
    }
    ~RayWindowExample() {
        UnloadRenderTexture(ViewTexture);
    }
    void Show() {
        ImGui::PushStyleVar(ImGuiStyleVar_WindowPadding, ImVec2(0, 0));
        ImGui::SetNextWindowSize(ImVec2(256, 275), ImGuiCond_FirstUseEver);
        if (ImGui::Begin("Raylib in ImGUI", &Open, ImGuiWindowFlags_NoScrollbar)) {
            Focused = ImGui::IsWindowFocused(ImGuiFocusedFlags_ChildWindows);
            ImVec2 DaZone = ImGui::GetContentRegionAvail();
            rlImGuiImageRenderTexture(&ViewTexture);
            // rlImGuiImage(&ViewTexture.texture);
            if (PastSize != DaZone) {
                Resized = true;
            } else {
                Resized = false;
            }
            PastSize = DaZone;
        }
        ImGui::End();
        ImGui::PopStyleVar();
    }
    void Update() {
        if (!Open) {
            return;
        }
        if (Resized) {
            UnloadRenderTexture(ViewTexture);
            ViewTexture = LoadRenderTexture(PastSize.x, PastSize.y);
        }

        BeginTextureMode(ViewTexture);
        ClearBackground(SKYBLUE);
        DrawCircle(0, 0, 35, RED);
        //  DrawPixel(0,0, BLUE);
        //  DrawRectangleLines(0,0,PastSize.x,PastSize.y, RED);
        Rectangle rec = Rectangle{0, 0, PastSize.x, PastSize.y};
        // DrawRectangleLinesEx(rec, 2, RED);
        EndTextureMode();
    }
};

std::vector<std::string> GetMprisServices() {
    auto connection = sdbus::createSessionBusConnection();

    sdbus::ServiceName dest{"org.freedesktop.DBus"};
    sdbus::ObjectPath objec{"/org/freedesktop/DBus"};

    auto prox = sdbus::createProxy(dest, objec);
    sdbus::InterfaceName interfaceName{"org.freedesktop.DBus"};
    sdbus::MethodName introspect("ListNames");

    auto method = prox->createMethodCall(interfaceName, introspect);
    auto reply = prox->callMethod(method);
    std::vector<std::string> res;
    reply >> res;

    std::vector<std::string> ret;

    for (std::string item : res) {
        if (item.find("org.mpris.MediaPlayer2") != std::string::npos) {
            ret.push_back(item);
        }
    }
    return ret;
}

void DrawPlay(Vector2 Pos, float scale, Color color) {
    Vector2 backtop = (Vector2){.x = -0.29f, .y = -0.40f};
    Vector2 backbot = (Vector2){.x = -0.29f, .y = 0.40f};
    Vector2 front = (Vector2){.x = 0.5f, .y = 0};

    backtop = Vector2Scale(backtop, scale);
    backbot = Vector2Scale(backbot, scale);
    front = Vector2Scale(front, scale);

    backtop = Vector2Add(backtop, Pos);
    backbot = Vector2Add(backbot, Pos);
    front = Vector2Add(front, Pos);

    DrawTriangle(backtop, backbot, front, color);
}

void DrawNext(Vector2 Pos, float scale, Color color) {
    Vector2 backtop = (Vector2){.x = -0.35f, .y = -0.35f};
    Vector2 backbot = (Vector2){.x = -0.35f, .y = 0.35f};
    Vector2 front = (Vector2){.x = 0.28f, .y = 0};

    backtop = Vector2Scale(backtop, scale);
    backbot = Vector2Scale(backbot, scale);
    front = Vector2Scale(front, scale);

    backtop = Vector2Add(backtop, Pos);
    backbot = Vector2Add(backbot, Pos);
    front = Vector2Add(front, Pos);

    DrawTriangle(backtop, backbot, front, color);

    Vector2 topleft = (Vector2){.x = -0.29f, .y = -0.4f};

    Vector2 bar2topleft = Vector2Add(topleft, (Vector2){0.706f * 0.6f, .y = 0});
    bar2topleft = Vector2Scale(bar2topleft, scale);
    bar2topleft = Vector2Add(bar2topleft, Pos);

    Vector2 size = (Vector2){.x = 0.706f * 0.2f, .y = 0.8f};
    size = Vector2Scale(size, scale);

    DrawRectangleV(bar2topleft, size, color);
}

void DrawBack(Vector2 Pos, float scale, Color color) {
    Vector2 backtop = (Vector2){.x = 0.35f, .y = -0.35f};
    Vector2 backbot = (Vector2){.x = 0.35f, .y = 0.35f};
    Vector2 front = (Vector2){.x = -0.28f, .y = 0};

    backtop = Vector2Scale(backtop, scale);
    backbot = Vector2Scale(backbot, scale);
    front = Vector2Scale(front, scale);

    backtop = Vector2Add(backtop, Pos);
    backbot = Vector2Add(backbot, Pos);
    front = Vector2Add(front, Pos);

    // DrawTriangle(backtop, backbot, front, color);
    DrawTriangle(front, backbot, backtop, color);

    Vector2 topleft = (Vector2){.x = -0.29f, .y = -0.4f};

    Vector2 bar1topleft = Vector2Scale(topleft, scale);
    bar1topleft = Vector2Add(bar1topleft, Pos);

    Vector2 size = (Vector2){.x = 0.706f * 0.2f, .y = 0.8f};
    size = Vector2Scale(size, scale);

    DrawRectangleV(bar1topleft, size, color);
}

void DrawStop(Vector2 Pos, float scale, Color color) {
    Vector2 topleft = (Vector2){.x = -0.353f, .y = -0.353f};
    Vector2 botright = (Vector2){.x = 0.353f, .y = 0.353f};
    topleft = Vector2Scale(topleft, scale);
    botright = Vector2Scale(botright, scale * 2);

    topleft = Vector2Add(topleft, Pos);

    DrawRectangleV(topleft, botright, color);
}

void DrawPause(Vector2 Pos, float scale, Color color) {
    Vector2 topleft = (Vector2){.x = -0.353f, .y = -0.353f};

    Vector2 bar1topleft = Vector2Scale(topleft, scale);
    bar1topleft = Vector2Add(bar1topleft, Pos);

    Vector2 bar2topleft = Vector2Add(topleft, (Vector2){0.706f * 0.6f, .y = 0});
    bar2topleft = Vector2Scale(bar2topleft, scale);
    bar2topleft = Vector2Add(bar2topleft, Pos);

    Vector2 size = (Vector2){.x = 0.706f * 0.4f, .y = 0.706f};
    size = Vector2Scale(size, scale);

    topleft = Vector2Add(topleft, Pos);

    DrawRectangleV(bar1topleft, size, color);
    DrawRectangleV(bar2topleft, size, color);
}

float ScaleToFit(Vector2 src, Vector2 dst) {
    float ratio = std::min(dst.x / src.x, dst.y / src.y);
    return ratio;
    // return Vector2Scale(src, ratio);
}

void GenCircleOverlay(MprisPlayer *current_player, RenderTexture tex, bool trans, bool accent) {
    BeginTextureMode(tex);
    if (trans) {
        ClearBackground(WHITE);
    } else {
        if (accent) {
            ClearBackground(current_player->GetPrimaryColor());
        } else {
            ClearBackground(BLACK);
        }
    }
    BeginBlendMode(BLEND_SUBTRACT_COLORS);
    if (trans) {
        DrawCircle(GetScreenWidth() / 2, GetScreenHeight() / 2, std::min(GetScreenWidth() / 2, GetScreenHeight() / 2), WHITE);
    } else {
        DrawCircle(GetScreenWidth() / 2, GetScreenHeight() / 2, std::min(GetScreenWidth() / 2, GetScreenHeight() / 2), (Color{0, 0, 0, 0}));
    }
    EndBlendMode();
    EndTextureMode();
}

int main(int argc, char *argv[]) {
    // Initialization
    //--------------------------------------------------------------------------------------
    int screenWidth = 800;
    int screenHeight = 800;

    // SetConfigFlags(FLAG_MSAA_4X_HINT | FLAG_WINDOW_RESIZABLE);
    // SetConfigFlags(FLAG_MSAA_4X_HINT | FLAG_VSYNC_HINT | FLAG_WINDOW_RESIZABLE);
    SetConfigFlags(FLAG_WINDOW_TRANSPARENT | FLAG_MSAA_4X_HINT | FLAG_WINDOW_RESIZABLE | FLAG_VSYNC_HINT);
    InitWindow(screenWidth, screenHeight, "raylib-Extras [ImGui] example - Docking");
    SetTargetFPS(244);
    rlImGuiSetup(true);
    ImGui::GetStyle().AntiAliasedLinesUseTex = false;
    // SetWindowState(FLAG_WINDOW_UNDECORATED);

    TestC();
    TestCPPFunc();

    curl_global_init(CURL_GLOBAL_ALL);

    bool run = true;
    bool showIMgui = false;

    bool showDemoWindow = false;

    std::vector<std::string> serviceList = GetMprisServices();

    auto current_player = MprisPlayer("org.mpris.MediaPlayer2.playerctld");
    double lasttime = 0.0;
    bool discmode = false;
    DiscObject disc(0);
    // Main game loop
    bool goodstate = false;
    bool trans = false;
    bool accent = true;

    long imagecount = -1;

    bool showMetricsWindow = false;

    RenderTexture recordoverlay = LoadRenderTexture(screenWidth, screenHeight);
    GenCircleOverlay(&current_player, recordoverlay, trans, accent);

    while (!WindowShouldClose() && run) // Detect window close button or ESC key, or a quit from the menu
    {
        if (IsWindowResized()) {
            UnloadRenderTexture(recordoverlay);
            recordoverlay = LoadRenderTexture(GetScreenWidth(), GetScreenHeight());
            GenCircleOverlay(&current_player, recordoverlay, trans, accent);
        }

        if (current_player.imagecount > imagecount) {
            imagecount = current_player.imagecount;
            GenCircleOverlay(&current_player, recordoverlay, trans, accent);
        }

        if (GetTime() > lasttime + 3) {
            lasttime = GetTime();
            current_player.Refresh();
        }

        BeginDrawing();
        if (trans) {
            ClearBackground((Color){0, 0, 0, 0});
        } else {
            ClearBackground(BLACK);
        }
        Vector2 winwidth = (Vector2){(float)GetScreenWidth(), (float)GetScreenHeight()};
        Vector2 imagesize = (Vector2){(float)current_player.tex.tex.width, (float)current_player.tex.tex.height};

        Vector2 scaledsize = Vector2Scale(imagesize, ScaleToFit(imagesize, winwidth));
        Vector2 fpos;
        fpos.x = (winwidth.x * 0.5) - (scaledsize.x * 0.5f);
        fpos.y = (winwidth.y * 0.5) - (scaledsize.y * 0.5f);
        // DrawTextureEx(current_player.tex.tex, fpos, 0,  ScaleToFit(imagesize, winwidth), WHITE);
        Rectangle src = (Rectangle){0, 0, (float)current_player.tex.tex.width, (float)current_player.tex.tex.height};
        Rectangle dst = (Rectangle){fpos.x + scaledsize.x / 2, fpos.y + scaledsize.y / 2, scaledsize.x, scaledsize.y};

        DrawTexturePro(current_player.tex.tex, src, dst, (Vector2){dst.width / 2, dst.height / 2}, disc.pos, WHITE);

        if (IsWindowFocused()) {
            const float buttonsize = 0.1;
            Vector2 screen = (Vector2){(float)GetScreenWidth(), (float)GetScreenHeight()};
            Vector2 centerline;
            centerline.x = screen.x / 2;
            centerline.y = screen.y * 0.8;

            Color transblack = BLACK;
            transblack.a = (200);
            DrawCircleV(centerline, 0.05 * GetScreenWidth(), transblack);

            if (current_player.playstate == "Playing") {
                DrawPause(centerline, 0.1 * GetScreenWidth(), WHITE);
            } else {
                DrawPlay(centerline, 0.1 * GetScreenWidth(), WHITE);
            }

            Vector2 next = Vector2Add(centerline, (Vector2){screen.x * 0.2f, 0.0f});
            DrawCircleV(next, 0.05 * GetScreenWidth(), transblack);
            DrawNext(next, 0.1 * GetScreenWidth(), WHITE);

            Vector2 back = Vector2Add(centerline, (Vector2){screen.x * -0.2f, 0.0f});
            DrawCircleV(back, 0.05 * GetScreenWidth(), transblack);
            DrawBack(back, 0.1 * GetScreenWidth(), WHITE);

            if (IsMouseButtonPressed(MOUSE_BUTTON_LEFT) && CheckCollisionPointCircle(GetMousePosition(), centerline, 0.05 * GetScreenWidth())) {
                current_player.PausePlay();
                current_player.playstate = current_player.playstate == "Playing" ? "Paused" : "Playing";
            } else if (IsMouseButtonPressed(MOUSE_BUTTON_LEFT) && CheckCollisionPointCircle(GetMousePosition(), next, 0.05 * GetScreenWidth())) {
                current_player.Next();
                current_player.playstate = current_player.playstate == "Playing" ? "Paused" : "Playing";
            } else if (IsMouseButtonPressed(MOUSE_BUTTON_LEFT) && CheckCollisionPointCircle(GetMousePosition(), back, 0.05 * GetScreenWidth())) {
                current_player.Prev();
                current_player.playstate = current_player.playstate == "Playing" ? "Paused" : "Playing";
            } else if (IsMouseButtonDown(MOUSE_BUTTON_LEFT) && disc.enabled) {
                float initmouserot;
                Vector2 initmousepos;
                if (IsMouseButtonPressed(MOUSE_BUTTON_LEFT)) {
                    initmouserot = disc.pos;
                    initmousepos = GetMousePosition();
                    initmousepos = Vector2Add(initmousepos, (Vector2){-screen.x / 2, -screen.y / 2});
                    goodstate = true;
                }
                if (goodstate) {
                    Vector2 mouse = GetMousePosition();
                    Vector2 mousecentered = Vector2Add(mouse, (Vector2){-screen.x / 2, -screen.y / 2});
                    disc.velo = 0;
                    disc.UpdatePos(initmouserot + (((180.0f * atan2(mousecentered.y, mousecentered.x)) / M_PI) - ((180.0f * atan2(initmousepos.y, initmousepos.x)) / M_PI)), GetFrameTime());
                }
            }
        }
        if (!goodstate) {
            disc.UpdatePos(GetFrameTime());
            // Ease from velo to target
            if (current_player.playstate == "Playing") {
                disc.target = 100;
            } else {
                disc.target = 0;
            }
            disc.accel = disc.target - disc.velo;
        }
        if (IsMouseButtonReleased(MOUSE_BUTTON_LEFT)) {
            goodstate = false;
        }

        if (disc.active) {
            if (trans) {
                rlSetBlendFactors(1, 1, 0x800B);
                BeginBlendMode(BLEND_CUSTOM);
            }
            DrawTexture(recordoverlay.texture, 0, 0, WHITE);
            if (trans) {
                EndBlendMode();
            }
        }

        if (IsKeyPressed(KEY_SLASH) || IsMouseButtonPressed(MOUSE_BUTTON_RIGHT)) {
            showIMgui = !showIMgui;
        }

        // start ImGui content

        if (showIMgui) {
            disc.enabled = false;
            rlImGuiBegin();

            // if you want windows to dock to the viewport, call this.
            // show a simple menu bar
            if (ImGui::BeginMainMenuBar()) {
                if (ImGui::BeginMenu("File")) {

                    if (ImGui::MenuItem("Quit"))
                        run = false;

                    ImGui::EndMenu();
                }

                if (ImGui::BeginMenu("Players")) {
                    for (auto item : serviceList) {
                        if (ImGui::MenuItem(item.c_str())) {
                            current_player = MprisPlayer(item);
                        }
                    }
                    if (ImGui::MenuItem("Refresh")) {
                        serviceList = GetMprisServices();
                    }
                    ImGui::EndMenu();
                }

                if (ImGui::BeginMenu("Window")) {
                    if (ImGui::MenuItem("Demo Window", nullptr, showDemoWindow))
                        showDemoWindow = !showDemoWindow;
                    if (ImGui::MenuItem("Record Spin !!", nullptr, disc.active)) {
                        disc.Activate();
                    }
                    if (ImGui::MenuItem("Trans Mode !!", nullptr, trans)) {
                        trans = !trans;
                        GenCircleOverlay(&current_player, recordoverlay, trans, accent);
                    }
                    if (ImGui::MenuItem("Accent Mode !!", nullptr, accent)) {
                        accent = !accent;
                        GenCircleOverlay(&current_player, recordoverlay, trans, accent);
                    }
                    ImGui::EndMenu();
                }
                if (ImGui::BeginMenu("Tools")) {
                    if (ImGui::MenuItem("Debug Window", nullptr, showMetricsWindow)) {
                        showMetricsWindow = !showMetricsWindow;
                    }
                    ImGui::EndMenu();
                }

                ImGui::EndMainMenuBar();

                if (showMetricsWindow) {
                    ImGui::ShowMetricsWindow();
                }
            }

            // show some windows

            if (ImGui::Begin("Player Control")) {
                ImGui::Text("%s", current_player.GetIdentity().c_str());

                if (ImGui::Button("Prev")) {
                    current_player.Prev();
                }
                ImGui::SameLine();
                if (ImGui::Button("Play/Pause")) {
                    current_player.PausePlay();
                }
                ImGui::SameLine();
                if (ImGui::Button("Next")) {
                    current_player.Next();
                }
                ImGui::SameLine();
                if (ImGui::Button("Refresh")) {
                    current_player.Refresh();
                    current_player.UpdateTexture();
                }

                // ImGui::SliderFloat("erm", &rotation, -180, 180, "Rotation");
                // rlImGuiImage(&current_player.tex.tex);
                ImGui::Separator();
                ImGui::LabelText("Disc Info", "P: %f, V: %f, A: %f", disc.pos, disc.velo, disc.accel);
                rlImGuiImageSize(&current_player.tex.tex, 300, 300);

                ImGui::Separator();
                int inc = 0;
                for (auto obj : current_player.accent_colors) {

                    float col[3] = {0, 0, 0};
                    col[0] = ((float)obj.r) / 255;
                    col[1] = ((float)obj.g) / 255;
                    col[2] = ((float)obj.b) / 255;
                    ImGui::ColorEdit3(("lf" + std::to_string(inc)).c_str(), col);
                    inc++;
                }
            }
            ImGui::End();

            if (showDemoWindow)
                ImGui::ShowDemoWindow(&showDemoWindow);

            // end ImGui Content
            rlImGuiEnd();
        } else {
            disc.enabled = true;
        }

        EndDrawing();
        //----------------------------------------------------------------------------------
    }
    rlImGuiShutdown();

    // De-Initialization
    //--------------------------------------------------------------------------------------
    CloseWindow(); // Close window and OpenGL context
    //--------------------------------------------------------------------------------------
    curl_global_cleanup();
    return 0;
}