#include "mpris_connection.hpp"
#include <sdbus-c++/IConnection.h>
#include <sdbus-c++/Types.h>
#include <string>
#define IMGUI_DEFINE_MATH_OPERATORS
#include "imgui.h"

#include <cpp/score/score.h>
#include <curl/curl.h>
#include <fstream>
#include <iostream>
#include <magic.h>
#include <ostream>
#include <unordered_map>

MprisPlayer::MprisPlayer(std::string servicename) {
    bus_connection = sdbus::createSessionBusConnection();
    sdbus::ServiceName dest{servicename};
    sdbus::ObjectPath objec{"/org/mpris/MediaPlayer2"};
    image_path_cache = "NULL";
    PlayerProxy = sdbus::createProxy(dest, objec);
    mp2 = "org.mpris.MediaPlayer2.Player";
    imagecount = 0;
    Refresh();
}

void MprisPlayer::PausePlay() {
    auto method = PlayerProxy->createMethodCall(mp2, sdbus::MethodName("PlayPause"));
    try {
        auto reply = PlayerProxy->callMethod(method);
    } catch (const std::exception& e) {}
}

void MprisPlayer::Next() {
    auto method = PlayerProxy->createMethodCall(mp2, sdbus::MethodName("Next"));
    try {
        auto reply = PlayerProxy->callMethod(method);
    } catch (const std::exception &e) {}
}

void MprisPlayer::Prev() {
    auto method = PlayerProxy->createMethodCall(mp2, sdbus::MethodName("Previous"));
    try {
        auto reply = PlayerProxy->callMethod(method);
    } catch (const std::exception &e) {}
}

std::string MprisPlayer::GetIdentity() {
    return identity;
}

std::unordered_map<std::string, sdbus::Variant> MprisPlayer::GetCurrentMetadata() {
    return current_metadata;
}
void MprisPlayer::Refresh() {
    try {
        identity = PlayerProxy->getProperty("Identity")
                       .onInterface("org.mpris.MediaPlayer2")
                       .get<std::string>();
        playstate = PlayerProxy->getProperty("PlaybackStatus").onInterface("org.mpris.MediaPlayer2.Player").get<std::string>();
        current_metadata = PlayerProxy->getProperty("Metadata")
                               .onInterface("org.mpris.MediaPlayer2.Player")
                               .get<std::unordered_map<std::string, sdbus::Variant>>();
        if (current_metadata["mpris:artUrl"].get<std::string>() != image_path_cache) {
            image_path_cache = current_metadata["mpris:artUrl"].get<std::string>();
            UpdateTexture();
            auto img = LoadImageFromTexture(tex.tex);

            std::vector <material_color_utilities::Argb> argbs;
            material_color_utilities::QuantizerResult res;
            if (img.format == PIXELFORMAT_UNCOMPRESSED_R8G8B8A8) {
                auto data = (char*) img.data;
                for (int i = 0; i < img.width * img.height * 4; i += 4) {
                    material_color_utilities::Argb color = (data[i] << 16) | (data[i + 1] << 8) | (data[i + 2]) | (data[i + 3] << 24);
                    argbs.push_back(color);
                }
                res = material_color_utilities::QuantizeCelebi(argbs, 16);
            } else if (img.format == PIXELFORMAT_UNCOMPRESSED_R8G8B8) {
                auto data = (char*) img.data;
                for (int i = 0; i < img.width * img.height * 3; i += 3) {
                    material_color_utilities::Argb color = (data[i] << 16) | (data[i + 1] << 8) | (data[i + 2] | (0xFF << 24));
                    argbs.push_back(color);
                }
                res = material_color_utilities::QuantizeCelebi(argbs, 16);
            }

            auto colors = material_color_utilities::RankedSuggestions(res.color_to_count, material_color_utilities::ScoreOptions());
            accent_colors.clear();
            for (auto obj : colors) {
                accent_colors.push_back((Color) {.r=(unsigned char) ((obj >> 16) & 0xFF), .g=(unsigned char)( (obj >> 8) & 0xFF  ), .b=(unsigned char)((obj >> 0) & 0xFF), .a=(unsigned char)((obj>>24) & 0xFF) });
            }
            imagecount = imagecount + 1;

        }
    } catch (const std::exception& e) {}
    return;
}


static size_t WriteMemoryCallback(void* contents, size_t size, size_t nmemb,
                                  void* userp) {
    size_t realsize = size * nmemb;
    auto& mem = *static_cast<std::string*>(userp);
    mem.append(static_cast<char*>(contents), realsize);
    return realsize;
}

void MprisPlayer::UpdateTexture() {
    std::string filename = image_path_cache;
    std::cout << filename << std::endl;
    if (filename.length() > 0) {
        if (filename.c_str()[0] == 'h') {
            CURL* curl_handle;
            CURLcode res;

            std::string chunk;


            curl_handle = curl_easy_init();
            curl_easy_setopt(curl_handle, CURLOPT_URL, filename.c_str());
            curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
            curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, &chunk);
            curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "libcurl-agent/1.0");

            // added options that may be required
            curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1L);  // redirects
            curl_easy_setopt(curl_handle, CURLOPT_HTTPPROXYTUNNEL, 1L); // corp. proxies etc.
            curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, 1L); // we want it all
            // curl_easy_setopt(curl_handle, CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS);

            res = curl_easy_perform(curl_handle);

            if(res != CURLE_OK) {
                std::cerr << "curl_easy_perform() failed: " << curl_easy_strerror(res) << '\n';
            } else {
                std::cout << chunk.size() << " bytes retrieved\n";
            }

            std::ofstream output("/tmp/mpris-miniplayer-out");

            if (output.is_open()) {
                output << chunk;
                output.close();
            }
            filename = "file:///tmp/mpris-miniplayer-out";

            curl_easy_cleanup(curl_handle);


        }
        if (filename.c_str()[0] == 'f') {
            std::cout << "parsing as file" << std::endl;
            std::string path = filename.substr(7);
            FILE * f = fopen(path.c_str(), "r");
            if (f == NULL) {
                tex = RayTexture();
                fclose(f);
                return;
            }
            fseek(f, 0, SEEK_END);
            long size = ftell(f);
            rewind(f);
            unsigned char* buffer = new unsigned char[size];
            fread(buffer, size, 1, f);

            magic_t magic = magic_open(MAGIC_MIME_TYPE);
            if (magic == NULL) {
                tex = RayTexture();
                delete[] buffer;
                fclose(f);
                return;
            }

            if (magic_load(magic, NULL) != 0) {
                tex = RayTexture();
                delete[] buffer;
                fclose(f);
                return;
            }

            const char *mime_type = magic_file(magic, path.c_str());
            if (mime_type == NULL) {
                tex = RayTexture();
                delete[] buffer;
                fclose(f);
                return;
            }

            std::cout << mime_type << std::endl;
            if (strcmp(mime_type, "image/png") == 0 ) {
                auto img = LoadImageFromMemory(".png", buffer, size);
                tex = RayTexture(img);
                UnloadImage(img);
            } else if (strcmp(mime_type, "image/jpeg") == 0 ) {
                auto img = LoadImageFromMemory(".jpg", buffer, size);
                tex = RayTexture(img);
                UnloadImage(img);
            } else {
                tex = RayTexture();
            }
                delete[] buffer;
                fclose(f);
                return;

        }
    }
}

Color MprisPlayer::GetPrimaryColor() {
    if (accent_colors.empty()) {
        return BLACK;
    } else {
        std::cout << "here" <<  std::endl;
        return accent_colors[0];
    }
}


DiscObject::DiscObject(float ia) {
    pos = 0;
    prevouspos = 0;
    velo = 0;
    accel = 0;
    target = ia;
    active = true;
    enabled = true;
}

void DiscObject::Activate() {
    active = !active;
    pos = 0;
    prevouspos = 0;
    velo = 0;
    accel = 0;
    target = 0;
}



void DiscObject::UpdatePos(float dtime) {
    if (active) {
        prevouspos = pos;
        pos = pos + velo * dtime;
        velo = velo + accel * dtime;
        accel = target;
    }
}

void DiscObject::UpdatePos(float new_pos, float dtime) {
    if (active) {
        prevouspos = pos;
        pos = new_pos;
        velo = (pos - prevouspos) / dtime;
        accel = target;
    }
}


void DiscObject::AddAccel(float ac) {
    accel = accel + ac;
}

 RayTexture::RayTexture() {
    auto rtex = LoadRenderTexture(300,300);
    BeginTextureMode(rtex);
    DrawRectangle(0,0,300,300,BLACK);
    EndTextureMode();
    auto img = LoadImageFromTexture(rtex.texture);
    tex = LoadTextureFromImage(img);
    UnloadImage(img);
    UnloadRenderTexture(rtex);
    GenTextureMipmaps(&tex);
    SetTextureFilter(tex, TEXTURE_FILTER_BILINEAR);
    path = "null";
}

 RayTexture::RayTexture(Image img) {
    path = "NOPATH";
    tex = LoadTextureFromImage(img);
    GenTextureMipmaps(&tex);
    SetTextureFilter(tex, TEXTURE_FILTER_BILINEAR);
}

 RayTexture::RayTexture(const RayTexture &other) {
    path = other.path;
    auto img = LoadImageFromTexture(other.tex);
    tex = LoadTextureFromImage(img);
    GenTextureMipmaps(&tex);
    SetTextureFilter(tex, TEXTURE_FILTER_BILINEAR);
    UnloadImage(img);
}

RayTexture &RayTexture::operator=(const RayTexture &other) {
    if (this == &other) {
        return *this;
    }
    printf("HERE\n");
    RayTexture temp(other);
    std::swap(tex, temp.tex);
    std::swap(path, temp.path);
    return *this;
}

 RayTexture::~RayTexture() {
    if (IsWindowReady()) {
        UnloadTexture(tex);
    }
}
